﻿Imports System.ComponentModel

Public Class CalcDpi

    'Subroutine CalcDpi_Load
    '- Actions at opening the form OziBuild
    Private Sub CalcDpi_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        TipText.SetToolTip(TopStrip, "With all four variables in place, the DPI can be calculated." & EndLine & "By changing the physical unit the DPI will be recalculated.")
        TipText.SetToolTip(UnitBox, "Select the unit of the phisical size of the image." & EndLine & "By changing the physical unit the DPI will be recalculated.")

        '- Disabling M4MM form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Calculate DPI"

        'Setting values for UnitBox (or related)
        UnitBox.Items.Add("mm")
        UnitBox.Items.Add("cm")
        UnitBox.Items.Add("inch")
        UnitBox.Text = "mm"
        PhyHeightLabel.Text = "Physical height in mm"
        PhyWidthLabel.Text = "Physical width in mm"

    End Sub

    'Subroutine CalcDpi_Closing
    '- Actions at closing the form CalcDpi
    Private Sub CalcDpi_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling M4MM form
        StartMenu.Enabled = True

    End Sub

    'Subroutine CloseItem_Click()
    '- Initializing closing the form OziBuild
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine PhyHeightValue_KeyPress
    '- Handles digits, control, dot and enter keystrokes
    Private Sub PhyHeightValue_KeyPress(sender As Object, e As KeyPressEventArgs) Handles PhyHeightValue.KeyPress
        If Not (Char.IsDigit(e.KeyChar) Or
                Char.IsControl(e.KeyChar) Or
                e.KeyChar = "." Or
                e.KeyChar = Chr(13)) Then
            e.KeyChar = ""

        Else
            'Only one dos is allowed
            If e.KeyChar = "." And InStr(PhyHeightValue.Text, ".", CompareMethod.Binary) > 0 Then
                e.KeyChar = ""
            End If
            'Enter key
            If e.KeyChar = Chr(13) Then
                PhyWidthValue.Focus()
            End If

        End If

    End Sub

    'Subroutine PhyWidthValue_KeyPress
    '- Handles digits, control, dot and enter keystrokes
    Private Sub PhyWidthValue_KeyPress(sender As Object, e As KeyPressEventArgs) Handles PhyWidthValue.KeyPress
        If Not (Char.IsDigit(e.KeyChar) Or
                Char.IsControl(e.KeyChar) Or
                e.KeyChar = "." Or
                e.KeyChar = Chr(13)) Then
            e.KeyChar = ""
        Else
            'Only one dos is allowed
            If e.KeyChar = "." And InStr(PhyWidthValue.Text, ".", CompareMethod.Binary) > 0 Then
                e.KeyChar = ""
            End If
            'Enter key
            If e.KeyChar = Chr(13) Then
                PixHeightValue.Focus()
            End If

        End If

    End Sub

    'Subroutine PixHeightValue_KeyPress
    '- Handles digits, control and enter keystrokes
    Private Sub PixHeightValue_KeyPress(sender As Object, e As KeyPressEventArgs) Handles PixHeightValue.KeyPress

        If Not (Char.IsDigit(e.KeyChar) Or
                Char.IsControl(e.KeyChar) Or
                e.KeyChar = Chr(13)) Then
            e.KeyChar = ""
        Else
            If e.KeyChar = Chr(13) Then
                PixWidthValue.Focus()
            End If

        End If

    End Sub

    'Subroutine PixwidthValue_KeyPress
    '- Handles digits, control and enter keystrokes
    Private Sub PixwidthValue_KeyPress(sender As Object, e As KeyPressEventArgs) Handles PixWidthValue.KeyPress

        If Not (Char.IsDigit(e.KeyChar) Or
                Char.IsControl(e.KeyChar) Or
                e.KeyChar = Chr(13)) Then
            e.KeyChar = ""
        Else
            If e.KeyChar = Chr(13) Then
                PhyHeightValue.Focus()
            End If

        End If

    End Sub

    'Subroutine CalculateItem_Click
    '- Calculates resolution if all four input items are in place
    '- Takes unit of physical size in account 
    Private Sub CalculateItem_Click(sender As Object, e As EventArgs) Handles CalculateItem.Click
        Dim HorRes As Double    'Horizontal resolution
        Dim VerRes As Double    'Vertical resolution
        Dim Res As Double       'Overall resolution

        '- Checking four inpunt values
        If Len(Trim(PhyHeightValue.Text)) > 0 And
                Len(Trim(PhyWidthValue.Text)) > 0 And
                Len(Trim(PixHeightValue.Text)) > 0 And
                Len(Trim(PixWidthValue.Text)) > 0 Then

            '- Calculating 
            HorRes = Val(PixWidthValue.Text) / Val(PhyWidthValue.Text)
            VerRes = Val(PixHeightValue.Text) / Val(PhyHeightValue.Text)

            '- the different unit'sizes'
            Select Case UnitBox.Text
                Case "mm"
                    HorRes = HorRes * 25.4
                    VerRes = VerRes * 25.4
                    Res = (HorRes + VerRes) / 2
                Case "cm"
                    HorRes = HorRes * 2.54
                    VerRes = VerRes * 2.54
                    Res = (HorRes + VerRes) / 2
                Case "inch"
                    Res = (HorRes + VerRes) / 2
                Case Else
                    HorRes = 0
                    VerRes = 0
                    Res = 0
            End Select

            '- Copying values to screen
            HorResValue.Text = HorRes
            VerResValue.Text = VerRes
            ResValue.Text = Res
        End If

    End Sub

    'Subroutine PhyHeightValue_TextChanged
    Private Sub PhyHeightValue_TextChanged(sender As Object, e As EventArgs) Handles PhyHeightValue.TextChanged
        '- Calling subroutine ClearResolutions
        ClearResolutions()

        '- Calling subroutine ChangeUnit to reset unit
        ChangeUnit()

    End Sub

    'Subroutine PhyWidthValue_TextChanged
    Private Sub PhyWidthValue_TextChanged(sender As Object, e As EventArgs) Handles PhyWidthValue.TextChanged
        '- Calling subroutine ClearResolutions
        ClearResolutions()

        '- Calling subroutine ChangeUnit to reset unit
        ChangeUnit()

    End Sub

    'Subroutine PixHeightValue_TextChanged
    Private Sub PixHeightValue_TextChanged(sender As Object, e As EventArgs) Handles PixHeightValue.TextChanged
        '- Calling subroutine ClearResolutions
        ClearResolutions()

        '- Calling subroutine ChangeUnit to reset unit
        ChangeUnit()

    End Sub

    'Subroutine PixWidthValue_TextChanged
    Private Sub PixWidthValue_TextChanged(sender As Object, e As EventArgs) Handles PixWidthValue.TextChanged
        '- Calling subroutine ClearResolutions
        ClearResolutions()

        '- Calling subroutine ChangeUnit to reset unit
        ChangeUnit()

    End Sub

    'Subroutine ClearResolutions
    '- Clearing values on screen
    Private Sub ClearResolutions()
        HorResValue.Text = ""
        VerResValue.Text = ""
        ResValue.Text = ""
    End Sub

    'Subroutine ChangeUnit
    '- Changing labes when unit for physical size is changed
    Private Sub ChangeUnit()
        Select Case UnitBox.Text
            Case "mm"
                PhyHeightLabel.Text = "Physical height in mm"
                PhyWidthLabel.Text = "Physical width in mm"
            Case "cm"
                PhyHeightLabel.Text = "Physical height in cm"
                PhyWidthLabel.Text = "Physical width in cm"
            Case "inch"
                PhyHeightLabel.Text = "Physical height in inch"
                PhyWidthLabel.Text = "Physical width in inch"
            Case Else
                MsgBox("unit for physical size isn't correct, choose from list.", 48, AppName & " - Warning")
        End Select

    End Sub

    'Subroutine UnitBox_TextChanged
    '- Handles the changing when an other unit is selected
    Private Sub UnitBox_TextChanged(sender As Object, e As EventArgs) Handles UnitBox.TextChanged
        'Calling subroutine CalculateItem_Click to recalculate resolution
        CalculateItem_Click(sender, e)

        'Calling subroutine ChangeUnit to reset unit
        ChangeUnit()

    End Sub

End Class