﻿Imports System.ComponentModel

Public Class MobacBuild

    'Variable declarations
    Public MapFile As FileNameStructure             'Variable; represents filename MAP-file.
    Public JprFile As FileNameStructure             'Variable; represents filename JPR-file.
    Public GeoData As String                        'Variable; represents the content of the MAP-file
    Public GeoDataLen As Integer                    'Variable; represents (text) length of GeoData
    Public GeoDataLine(60) As String                'Array; represents the content of each line in GeoData (will be (re)dimensioned for every map)
    Public GeoDataLineCount As Integer              'Variable; represents the amount of lines in GeoDataLineContent (will be (re)calculated for every map)
    Public GeoDataLineField(4) As String            'Array; reprents the content of a field in a GeoDataLine (used to extract coordinates)
    Public JprData As JprStructure                  'Variable; represents the JPR data
    Public ReferencePoint(4) As Coordinate          'Reference points for calibration and boundary (topleft, topright, bottomright, bottomleft)

    'Subroutine MobacBuild_Load
    '- Actions at opening the form MobacBuild
    Private Sub MobacBuild_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "The workflow: 1. Select a file, 2. build the JPR-file and 3. Save the JPR-file.")
        TipText.SetToolTip(GeoDataBox, "Content of Mobac MAP-file.")
        TipText.SetToolTip(GeoDataBox, "Content for JPR-file.")

        '- Clearing all values in form
        FileValue.Text = ""
        GeoDataBox.Text = ""
        JprDataBox.Text = ""

        '- Adding values to DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")
        'Default setting
        DpiBox.Text = "254"

        '- Adding values to ScaleBox
        ScaleBox.Items.Add("12500")
        ScaleBox.Items.Add("20000")
        ScaleBox.Items.Add("25000")
        ScaleBox.Items.Add("30000")
        ScaleBox.Items.Add("33333")
        ScaleBox.Items.Add("40000")
        ScaleBox.Items.Add("50000")
        ScaleBox.Items.Add("66666")
        ScaleBox.Items.Add("100000")
        ScaleBox.Items.Add("200000")
        ScaleBox.Items.Add("250000")
        ScaleBox.Items.Add("300000")
        ScaleBox.Items.Add("500000")
        'Default setting
        ScaleBox.Text = "50000"

        '- Disabling StartMenu form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Build JPR-file from Mobac MAP-file"

        '- Disabling TopStrip items (until item can be used)
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Disabling groups
        SelectGroup.Enabled = False
        JprGroup.Enabled = False

        '- Disabling Print menu items
        PrintItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form MobacBuild
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine MobacBuild_Closing
    '- Actions at closing the form MobacBuild
    Private Sub MobacBuild_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling M4MM form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectButton_Click
    '- Selecting Ozi MAP-file
    '- Extractig data from MAP-file
    '- Writing data and filename to the form
    '- Creating values for variables: MapFile and JprFile
    '- Breaking content of MAP-file in to lines and  load them in an array (GeoDataLine)
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click
        'Declarations
        Dim LinePointer As Integer                  'Subroutine internal pointer used As line indicator In GeoDAta
        Dim SelectFile As New OpenFileDialog()      'Creating a File dialog object 

        '- Clearing all values in form and disabling groups and TopStrip items
        FileValue.Text = ""
        GeoDataBox.Text = ""
        JprDataBox.Text = ""
        JprGroup.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Clearing all variables
        GeoData = ""
        JprData = Nothing

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "Mobac MAP-file(*.map)|*.map"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling menu items
            BuildItem.Enabled = False
            SaveItem.Enabled = False
            FileValue.Text = ""
            PrintItem.Enabled = False

            Exit Sub

        End If

        '- Setting Sourcefile
        MapFile = SplitFileName(SelectFile.FileName)

        '- Showing selected filename in form (FileValue)
        FileValue.Text = MapFile.LongName

        '- Setting Value for GeoData and writing these data to form
        GeoData = My.Computer.FileSystem.ReadAllText(SelectFile.FileName)
        GeoDataLen = Len(GeoData)

        '-  Showing Content of MapFile in GeoDataBox
        GeoDataBox.Text = GeoData

        '- setting values for JprFile
        JprFile = SplitFileName(MapFile.Path & MapFile.ShortName & ".jpr")

        '- Clearing JprData
        ClearJprData()

        'Breaking GeoData into Lines (in array GeoDataLine)
        '- Determening amount of lines (GeoDataLine)

        Dim TempLine() As String = Split(GeoData, EndLine)
        GeoDataLineCount = TempLine.Length - 1           'Determening number of lines

        ReDim GeoDataLine(GeoDataLineCount)

        For LinePointer = 0 To GeoDataLineCount - 1
            GeoDataLine(LinePointer + 1) = TempLine(LinePointer)
        Next

        '- Preventing crash in case LineCounter <> 56
        If GeoDataLineCount <> 56 Then
            MsgBox("This MAP-file wasn't created with Mobac.", 48, AppName & " - Warning")
            SelectItem.Select()
        Else
            '- Enabling next step (Building JPR-data)
            BuildItem.Enabled = True

            '- Locate cursor on BuildItem
            BuildItem.Select()
        End If

        '- Enabling next steps in the application and moving text cursor to GeoDataBox
        SelectGroup.Enabled = True
        GeoDataBox.Select()
        GeoDataBox.DeselectAll()
        BuildItem.Enabled = True
        BuildItem.Select()

        '- Enabling Print menu items
        PrintItem.Enabled = True

    End Sub

    'Subroutine BuildItem_Click
    '- Retreiving all data for JPR File
    '- Writing all data to JprFileBox in the correct structure
    Private Sub BuildItem_Click(sender As Object, e As EventArgs) Handles BuildItem.Click

        '- Building Content for JPR-File bij using function BuildJprContent
        JprDataBox.Text = BuildJprContent("ini")
        SaveItem.Select()

        '- Enabling next step in the application and moving text cursor to DataBox
        JprGroup.Enabled = True
        JprDataBox.Select()
        JprDataBox.DeselectAll()
        SaveItem.Enabled = True
        SaveItem.Select()

    End Sub

    'Function BuildJprContent generates the content for the JPR-file (as text string)
    '- Trigger is used to make teh difference between the initiator if teh function
    Private Function BuildJprContent(Trigger As String)
        '- Declarations
        Dim LinePointer As Integer                  'Variable; represents an array pointer
        Dim ContentText As String                   'Subroutine internal variable; represents the content for the JPR-File

        '- Retrieving calibrationpoints (ReferencePoint) and Size of a pixel in meter (JprData.ds)
        '  Boundary points (Ozi moving map points) wil be used as reference points
        For LinePointer = 41 To GeoDataLineCount
            Call SplitLine(GeoDataLine(LinePointer))
            Select Case LCase(GeoDataLineField(1))
                Case "mmpll"        'latitude and and longitude
                    ReferencePoint(Val(GeoDataLineField(2))).lon = Trim(GeoDataLineField(3))
                    ReferencePoint(Val(GeoDataLineField(2))).lat = Trim(GeoDataLineField(4))
                Case "mmpxy"        'pixel X and Y 
                    ReferencePoint(Val(GeoDataLineField(2))).x = Trim(GeoDataLineField(3))
                    ReferencePoint(Val(GeoDataLineField(2))).y = Trim(GeoDataLineField(4))
                Case "mm1b"         'Size of pixel in real in meter
                    JprData.ds = Val(GeoDataLineField(2))
            End Select
        Next

        '- Handling resolution and scale
        Select Case Trigger
            Case "ini"
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                'Writing dat to ScaleBox
                ScaleBox.Text = JprData.sc

            Case "dpi"
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                'Writing dat to ScaleBox
                ScaleBox.Text = JprData.sc

            Case "scale"
                JprData.sc = Val(ScaleBox.Text)
                JprData.sr = Int(JprData.sc * 2.54 / (JprData.ds * 100))

                'Writing dat to ScaleBox
                DpiBox.Text = JprData.sr

        End Select

        'Setting the data for variable JprData
        JprData.nm = LCase(Trim(GeoDataLine(2)))
        JprData.dm = "wgs84"
        JprData.st = 0
        JprData.sn = 0
        JprData.pr = "mercator"
        JprData.pp = "0"
        JprData.p2 = "1"
        JprData.p3 = "0"
        JprData.p4 = "0"
        JprData.it = "png"

        'Writing the data to 
        ContentText = "//This file was created using M4MM " & Version & " (Maps for Memory Map) by www.hzns.nl" & EndLine &
            "nm=" & JprData.nm & EndLine &
            "dm=" & JprData.dm & EndLine &
            "st=" & JprData.st & EndLine &
            "sn=" & JprData.sn & EndLine &
            "pr=" & JprData.pr & EndLine &
            "pp=" & JprData.pp & EndLine &
            "p2=" & JprData.p2 & EndLine &
            "p3=" & JprData.p3 & EndLine &
            "p4=" & JprData.p4 & EndLine &
            "it=" & JprData.it & EndLine &
            "sr=" & JprData.sr & EndLine &
            "sc=1:" & JprData.sc & EndLine &
            "rp1=" & ReferencePoint(1).lat & "," & ReferencePoint(1).lon & "," & ReferencePoint(1).x & "," & ReferencePoint(1).y & EndLine &
            "rp2=" & ReferencePoint(2).lat & "," & ReferencePoint(2).lon & "," & ReferencePoint(2).x & "," & ReferencePoint(2).y & EndLine &
            "rp3=" & ReferencePoint(3).lat & "," & ReferencePoint(3).lon & "," & ReferencePoint(3).x & "," & ReferencePoint(3).y & EndLine &
            "rp4=" & ReferencePoint(4).lat & "," & ReferencePoint(4).lon & "," & ReferencePoint(4).x & "," & ReferencePoint(4).y & EndLine &
            "vp1=" & ReferencePoint(1).x & "," & ReferencePoint(1).y & EndLine &
            "vp2=" & ReferencePoint(2).x & "," & ReferencePoint(2).y & EndLine &
            "vp3=" & ReferencePoint(3).x & "," & ReferencePoint(3).y & EndLine &
            "vp4=" & ReferencePoint(4).x & "," & ReferencePoint(4).y & EndLine

        '- Setting return value
        Return ContentText

    End Function

    'Subroutine SaveItem_Click 
    '-  Saving the JrpDataBox.text to a JPR-file in the same directory as the orginal MAP file. 
    '   If this JPR file exists a warning will be prompted.
    Private Sub SaveItem_Click(sender As Object, e As EventArgs) Handles SaveItem.Click
        '- Declarations
        Dim Answer As Integer       'Used as answer value for MsgBox

        '- Checking for an existing file
        If My.Computer.FileSystem.FileExists(JprFile.LongName) = True Then
            '(Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
            Answer = MsgBox("JPR-file allready exits. Overwrite?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox = Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
            End If
        Else
            'Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
        End If

    End Sub

    'Subroutine DipBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine DipBox_PressKey handles the case "Enter"-key was used (with focus on DpiBox)
    '- recalculaing the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine ScaleBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_LostFocus(sender As Object, e As EventArgs) Handles ScaleBox.LostFocus, ScaleBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()
            End If
        End If

    End Sub

    'Subroutine ScaleBox_PressKey handles the case "Enter"-key was used (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles ScaleBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine SplitLine splits a line with comma separeted fields into max 20 fields
    Private Sub SplitLine(LineText As String)
        '- Declarations
        Dim FieldPointer As Integer     'Variable; represents order of a field in a line
        Dim ChrPointer As Integer       'Variable; represents the place of a character in a string
        Dim TextChr As String           'Variable; represents a character

        '- Clear all information Array GeoDataLineField
        For FieldPointer = 0 To 4
            GeoDataLineField(FieldPointer) = ""
        Next

        '- Splitting line into fields (comma is separator: Chr=44)
        '  and writing to GeoDataLineField array
        FieldPointer = 1
        ChrPointer = 1
        Do While ChrPointer <= Len(LineText)
            Do
                TextChr = Mid(LineText, ChrPointer, 1)
                If TextChr <> Chr(44) Then
                    GeoDataLineField(FieldPointer) = GeoDataLineField(FieldPointer) + TextChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TextChr = Chr(44) Or ChrPointer > Len(LineText)
            If TextChr = Chr(44) Then FieldPointer = FieldPointer + 1
        Loop
    End Sub

    'Subroutine PrintDataItem_Click
    '- Initiates Printing GeoDataBox.text with color data
    Private Sub PrintDataItem_Click(sender As Object, e As EventArgs) Handles PrintDataItem.Click

        StartMenu.PrintContent(GeoDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine JprDataItem_Click
    '- Initiates Printing JprDataBox.text
    Private Sub JprDataItem_Click(sender As Object, e As EventArgs) Handles JprDataItem.Click

        StartMenu.PrintContent(JprDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine PrintItem_Click
    '- Testing if there is something to print (ExtractDataBox and JprDataBox)
    Private Sub PrintItem_Click(sender As Object, e As EventArgs) Handles PrintItem.Click


        '- Enabling/disabling printing JprDataBox
        If Trim(JprDataBox.Text) = "" Then
            JprDataItem.Enabled = False
        Else
            JprDataItem.Enabled = True
        End If

    End Sub

End Class