﻿Imports System.ComponentModel

Public Class ConvOzi

    'Declarations
    Public MapFile As FileNameStructure             'Variable; represents the MAP-filename (with GeoData)
    Public SourceFile As FileNameStructure          'Variable; represents the source-filename (image)
    Public TargetFile As FileNameStructure          'Variable; represents the target-filename (result image)

    'Subroutine OziConv_Load
    '- Actions at opening the form OziConv
    Private Sub ConvOzi_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "Select a Ozi MAP-file to view the content. If a file is selected the corresponding OZF2/OZFX3- file can be converted.")
        TipText.SetToolTip(MapFileValue, "Filename and location Ozi MAP-file.")
        TipText.SetToolTip(CommentBox, "Information about the coversion process.")
        TipText.SetToolTip(TargetFileValue, "Filename and location for the converted OZF2/OZFX3-file.")
        TipText.SetToolTip(BottomStrip, "Used Gdal version.")

        '- Clearing all values in form
        MapFileValue.Text = ""
        TargetFileValue.Text = ""
        CommentBox.Text = ""

        '- Disabling M4MM form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Convert Ozi-file"

        '- Disabling TopStrip items (until item can be used)
        ConvertItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine ConvOzi_Closing
    '- Actions at closing the form ConvOzi
    Private Sub ConvOzi_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling StartMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form ConvOzi
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine SelectItem_Click
    '- Selects a Ozi MAP-file 
    '- Checks for a corresponding OZF2/OZFX3-file
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click

        '- Declarations
        Dim MapFileContent As String                'Subroutine internal variable; represents (text) content of TempFileName
        Dim TempFile As FileNameStructure           'Subroutine internal variable; represents a filename

        '- Clearing all values in form and disabling groups and TopStrip items
        MapFileValue.Text = ""
        TargetFileValue.Text = ""
        CommentBox.Text = ""
        GdalVersion.Text = "GDAL version"

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "Ozi MAP-file (*.map)|*.map"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            ConvertItem.Enabled = False
            MsgBox("No Ozi MAP-file was selected!", 48, AppName & " - Warning")
            CommentBox.Text = CommentBox.Text & "No Ozi MAP-file was selected." & EndLine

            Exit Sub

        End If

        '- Retrieving values for MapFile 
        MapFile = SplitFileName(SelectFile.FileName)

        '- Setting value MapFileValue
        MapFileValue.Text = MapFile.LongName

        '- Looking for OZF2/OZFX3 files

        '- Extracting filename of SourceFile from MAP-file
        '  - Reading text form MAP-file
        MapFileContent = My.Computer.FileSystem.ReadAllText(MapFile.LongName)

        '  - Splitting MAP-file in to lines
        Dim TempText() As String = Split(MapFileContent, EndLine)

        '  - This step "between" is to extract the filename of the OZI-file (sometimes there is an 'unknown' path in MAP-File data)
        TempFile = SplitFileName(TempText(2))

        '  - Checking Extention for SourceFile (it must be an ozf file)
        If LCase(Strings.Left(TempFile.Ext, 3)) <> "ozf" Then
            ConvertItem.Enabled = False
            MsgBox("Matching file " & TempFile.Name & " isn't an OZI-file!", 48, AppName & " - Warning")
            CommentBox.Text = CommentBox.Text & "Matching file " & TempFile.Name & " isn't an OZI-file." & EndLine
            Exit Sub

        End If

        '- Building SourceFile
        SourceFile = SplitFileName(MapFile.Path & TempFile.Name)

        '- Finding OZI-file

        '- Handling if OZI-file doesn't exits
        If My.Computer.FileSystem.FileExists(SourceFile.LongName) = False Then
            ConvertItem.Enabled = False
            MsgBox("Matching OZI-file " & SourceFile.Name & " is not found!", 48, AppName & " - Warning")
            CommentBox.Text = CommentBox.Text & "Matching OZI-file " & SourceFile.Name & " is not found." & EndLine
            Exit Sub

        End If

        '- Writing information to the CommentBox
        CommentBox.Text = CommentBox.Text & "Corresponding OZI-file " & SourceFile.Name & " was found." & EndLine

        '- Setting values for TargetFile
        TargetFile = SplitFileName(SourceFile.Path & SourceFile.ShortName & ".png")

        TargetFileValue.Text = TargetFile.LongName

        '- Enabling next step
        ConvertItem.Enabled = True

    End Sub

    'Subroutine ConvertItem_Click converts OZF2/OZFX3-file to TIFF-file
    Private Sub ConvertItem_Click(sender As Object, e As EventArgs) Handles ConvertItem.Click
        '- Declarations     
        Dim CommandLine As String        'Subroutine internal variable; represents string to be executed shell commandline

        '- Building the CommandLine
        CommandLine = BatFile.LongName & " Gdal_Translate " &
            C34 & MapFile.LongName & C34 & " " &
            C34 & TargetFile.LongName & C34

        '- Executing the CommandLine for converting Ozi file
        Call Shell(CommandLine, AppWinStyle.Hide, Wait:=True)

        '- Writing information to the CommentBox
        CommentBox.Text = CommentBox.Text & "PNG-File (" & TargetFile.Name & ") from " & SourceFile.Name & " was created." & EndLine

        '- Retrieving used GDAL version using function UsedGdal and writing to GdalVersion in BottomStrip
        GdalVersion.Text = "Used GDAL version: " & UsedGdal()

    End Sub

End Class