﻿Imports System.ComponentModel

Public Class SidBuild

    '- Variable declarations
    Public SourceFile As FileNameStructure          'Variable; represents the source-filename (which contains WKT text)
    Public JprFile As FileNameStructure             'Variable; represents the JPR-filename
    Public ImageFile As FileNameStructure           'Variable; represents an image-filename 
    Public GeoData As String                        'Variable; represents the WKT content of the source-file
    Public GeoDataLen As Integer                    'Variable; represents the Length of GoeData-string
    Public ResData As ResStructure                  'Variable; represents the resolution-data
    Public JprData As JprStructure                  'Variable; represents the JPR data
    Public ReferencePoint(5) As Coordinate          'Array; represents five calibration points (topleft, topright, bottomright, bottomleft, center)
    Public VertexPoint(4) As Coordinate             'Array; represents the points of the boundary (topleft, topright, bottomright, bottomleft)
    Public VertexAmount As Integer = 4              'Variable; represents the amount of boundary points (initialy 4)
    Public NeatLine As String                       'Variable; represents the NEATLINE
    Public ImageData As ImageDataStructure          'Variable; represents extracted data from ImageFile

    'Subroutine SidBuild_Load
    '- Actions at opening the form SidBuild
    Private Sub SidBuild_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "The workflow: 1. Select a file, 2. extract the data, 3. build the JPR-file and 4. Save the JPR-file.")
        TipText.SetToolTip(GeoDataBox, "WKT content of MrSID-file.")
        TipText.SetToolTip(FileValue, "Filename and path of selected MrSID-file.")
        TipText.SetToolTip(ExtractDataBox, "Comment on and extracted data from MrSID-file.")
        TipText.SetToolTip(JprDataBox, "Content for JPR-file.")

        '- Clearing all values in form
        FileValue.Text = ""
        GeoDataBox.Text = ""
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""

        '- Adding values to DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")
        'Default setting
        DpiBox.Text = "254"

        '- Adding values to ScaleBox
        ScaleBox.Items.Add("12500")
        ScaleBox.Items.Add("20000")
        ScaleBox.Items.Add("25000")
        ScaleBox.Items.Add("30000")
        ScaleBox.Items.Add("33333")
        ScaleBox.Items.Add("40000")
        ScaleBox.Items.Add("50000")
        ScaleBox.Items.Add("66666")
        ScaleBox.Items.Add("100000")
        ScaleBox.Items.Add("200000")
        ScaleBox.Items.Add("250000")
        ScaleBox.Items.Add("300000")
        ScaleBox.Items.Add("500000")
        'Default setting
        ScaleBox.Text = "50000"

        '- Disabling StartMenu form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Build JPR-file from MrSID-file"

        '- Disabling TopStrip items (until item can be used)
        ExtractItem.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Disabling groups
        SelectGroup.Enabled = False
        ExtractGroup.Enabled = False
        JprGroup.Enabled = False

        '- Disabling Print menu items
        PrintItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form SidBuild
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine SidBuild_Closing
    '- Actions at closing the form SidBuild
    Private Sub SidBuild_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling StartMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectItem_Click
    '- Selecting and opening a Source-file
    '- Creating values for variables: SourceFile,  JprFile, ImageFile , GeoData
    '- Retrieving WKT from Source-file
    '- Presenting data on the screen (FileValue and GeoDataBox)
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click
        '- Declarations
        Dim ChrPointer As Integer                   'Subroutine internal variable; Point to a specific character in a string
        Dim TempDataFileName As String              'Subroutine internal variable; represents a (temp) filename (with path) (WKT text) 

        '- Clearing all values in form and disabling groups and TopStrip items
        FileValue.Text = ""
        GeoDataBox.Text = ""
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""
        ExtractItem.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "MrSID-file (*.sid)|*.sid"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling menu items
            ExtractItem.Enabled = False
            BuildItem.Enabled = False
            SaveItem.Enabled = False
            FileValue.Text = ""
            PrintItem.Enabled = False

            Exit Sub

        End If

        '- Setting Sourcefile (with WTK data)
        SourceFile = SplitFileName(SelectFile.FileName)

        '- Showing selected filename in form (FileValue)
        FileValue.Text = SourceFile.LongName

        '- Retrieving Geo-information (WKT 2.0 with GdalInfo and writing these data to GdalInfoBox)
        GeoData = WktText(SourceFile.LongName)

        '  - Loading GeoData into 'GeoDataBox'
        GeoDataBox.Text = GeoData

        '- Setting filename for file with JPR data
        JprFile = SplitFileName(SourceFile.Path & SourceFile.ShortName & "." & "jpr")

        '- Looking for a corresponding imagefile (to handle type of imagefile)

        '- Resettting ImageFile
        ImageFile = ClearFileName()

        '- looking for PNG-file
        TempDataFileName = SourceFile.Path & SourceFile.ShortName & "." & "png"
        If My.Computer.FileSystem.FileExists(TempDataFileName) = True Then
            ImageFile = SplitFileName(TempDataFileName)
        End If

        '- looking for TIF-file
        TempDataFileName = SourceFile.Path & SourceFile.ShortName & "." & "tif"
        If My.Computer.FileSystem.FileExists(TempDataFileName) = True Then
            ImageFile = SplitFileName(TempDataFileName)
        End If

        '- looking for TIFF-file
        TempDataFileName = SourceFile.Path & SourceFile.ShortName & "." & "tiff"
        If My.Computer.FileSystem.FileExists(TempDataFileName) = True Then
            ImageFile = SplitFileName(TempDataFileName)
        End If

        '- Testing for Geographic data (looking for the keyword 'Coordinate System is:')
        '  - If not: Warning
        '  - If so: Enabling next step
        ChrPointer = InStr(GeoData, "Coordinate System is:", CompareMethod.Binary)

        If ChrPointer = 0 Then
            MsgBox("The MrSID-file doesn't contain geographic data!", 48, AppName & " - Warning")
            '-Disabling next steps in the application
            ExtractItem.Enabled = False
            BuildItem.Enabled = False
            SaveItem.Enabled = False
            ExtractGroup.Enabled = False
            JprGroup.Enabled = False
            '- Enabling SelectGroup
            SelectGroup.Enabled = True
            GeoDataBox.Select()
            GeoDataBox.DeselectAll()
            Exit Sub

        End If

        '- Enabling next steps in the application and moving text cursor to GeoDataBox
        SelectGroup.Enabled = True
        GeoDataBox.Select()
        GeoDataBox.DeselectAll()
        ExtractItem.Enabled = True
        ExtractItem.Select()

        '- Enabling Print menu items
        PrintItem.Enabled = True

    End Sub

    'Sub ExtractItem_Click is retrieving/extracting information to create a JPR file
    Private Sub ExtractItem_Click(sender As Object, e As EventArgs) Handles ExtractItem.Click
        '- Declarations
        Dim Block(10) As BlockStructure                                 'Subroutine internal array; reprsents FirstLine and LastLine of blocks
        Dim GeoDataLine() As String = Split(GeoData, EndLine)           'Subroutine internal array; splitting GeoData in to lines
        Dim LineCount As Integer = GeoDataLine.Length - 1               'Subroutine internal variable; number of lines in array GeoDataLine
        Dim Pointer As Integer                                          'Subroutine internal variable; used as an array pointer
        Dim LinePointer As Integer                                      'Subroutine internal variable; points to a specific line
        Dim ChrPointer As Integer                                       'Subroutine internal variable; points to a specific charactaer in a string
        Dim TempString As String                                        'Subroutine internal variable; represents a string
        Dim Answer As Integer                                           'Subroutine internal variable; represents the return value of a MsgBox 
        Dim OrderCounter As Integer                                     'Subroutine internal variable; counts the number of [ORDER] is metric.
        Dim XResolution As Single                                       'Subroutine internal variable; represents the X resolution extracted for an image(file)
        Dim YResolution As Single                                       'Subroutine internal variable; represents the y resolution extracted for an image(file)

        '- Clearing all data
        Call ClearData()
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""
        DpiBox.Enabled = False

        '- Breaking GeoData in to blocks (to handle data and prevent "doulbe" values) 
        If InStr(GeoData, "BOUNDCRS", CompareMethod.Binary) > 0 Then
            '- Block 0, Block 1 (with subbocks 2,3,4) block 5, block 6, block 7 and Block 8
            Block(0).FirstLine = 0
            For LinePointer = 0 To LineCount

                If InStr(GeoDataLine(LinePointer), "Coordinate System is:", CompareMethod.Binary) > 0 Then
                    Block(1).FirstLine = LinePointer
                    Block(0).LastLine = Block(1).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "SOURCECRS", CompareMethod.Binary) > 0 Then
                    Block(2).FirstLine = LinePointer
                End If

                If InStr(GeoDataLine(LinePointer), "TARGETCRS", CompareMethod.Binary) > 0 Then
                    Block(3).FirstLine = LinePointer
                    Block(2).LastLine = Block(3).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "ABRIDGEDTRANSFORMATION", CompareMethod.Binary) > 0 Then
                    Block(4).FirstLine = LinePointer
                    Block(3).LastLine = Block(4).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Data axis to CRS", CompareMethod.Binary) > 0 Then
                    Block(5).FirstLine = LinePointer
                    Block(4).LastLine = Block(5).FirstLine - 1
                    Block(1).LastLine = Block(5).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Metadata:", CompareMethod.Binary) > 0 And
                        InStr(GeoDataLine(LinePointer), "Structure Metadata:", CompareMethod.Binary) = 0 Then
                    Block(6).FirstLine = LinePointer
                    Block(5).LastLine = Block(6).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Corner Coordinates:", CompareMethod.Binary) > 0 Then
                    Block(7).FirstLine = LinePointer
                    Block(6).LastLine = Block(7).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Band 1 Block=", CompareMethod.Binary) > 0 Then
                    Block(8).FirstLine = LinePointer
                    Block(7).LastLine = Block(8).FirstLine - 1
                End If
            Next
            Block(8).LastLine = LineCount

        Else
            '- Block 0, Block 2, block 5, block 6, block 7 and Block 8
            Block(0).FirstLine = 0
            For LinePointer = 0 To LineCount

                If InStr(GeoDataLine(LinePointer), "Coordinate System is:", CompareMethod.Binary) > 0 Then
                    Block(2).FirstLine = LinePointer
                    Block(0).LastLine = Block(2).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Data axis to CRS", CompareMethod.Binary) > 0 Then
                    Block(5).FirstLine = LinePointer
                    Block(2).LastLine = Block(5).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Metadata:", CompareMethod.Binary) > 0 And
                        InStr(GeoDataLine(LinePointer), "Structure Metadata:", CompareMethod.Binary) = 0 Then
                    Block(6).FirstLine = LinePointer
                    Block(5).LastLine = Block(6).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Corner Coordinates:", CompareMethod.Binary) > 0 Then
                    Block(7).FirstLine = LinePointer
                    Block(6).LastLine = Block(7).FirstLine - 1
                End If

                If InStr(GeoDataLine(LinePointer), "Band 1 Block=", CompareMethod.Binary) > 0 Then
                    Block(8).FirstLine = LinePointer
                    Block(7).LastLine = Block(8).FirstLine - 1
                End If
            Next
            Block(8).LastLine = LineCount

        End If

        '- Retrieving Nm (name of map)
        JprData.nm = JprFile.ShortName
        ExtractDataBox.Text = ExtractDataBox.Text & "nm (chartname) = " & JprData.nm & EndLine

        JprData.it = "sid"

        '- Screening the imagefile (if exists) and retrieving it(Image type; SID Or extention Of corresponding image file)

        ImageData = CreateImageData(ImageFile.LongName)     'Testing for and eventualy extrating image data 

        If ImageData.Exists = True Then                     'Image file is found

            If ImageData.Valid = False Then                 'No data could extracted from image file
                ExtractDataBox.Text = ExtractDataBox.Text & "A corresponding imagefile (" & ImageFile.LongName & ") is to large to extract data." & EndLine

            Else
                Answer = MsgBox("A corresponding imagefile (" & ImageFile.LongName & ") exits. Use data from this file?", 36, AppName & " - Question")

                If Answer = 6 Then                                          '6 = yes
                    ExtractDataBox.Text = ExtractDataBox.Text & "The corresponding imagefile will be used (" & ImageFile.Name & ")." & EndLine

                    JprData.it = ImageFile.Ext

                    '- Handling  colordepth
                    JprData.cd = ImageData.ColorDepth

                    If ImageData.Usable = False Then
                        ExtractDataBox.Text = ExtractDataBox.Text & "The colordepth `(" & JprData.cd & ") isn't compatible with Memory-Map" & EndLine
                    End If

                End If

            End If

        End If

        ExtractDataBox.Text = ExtractDataBox.Text & "it (graphics type) = " & JprData.it & EndLine

        If ImageFile.Ext <> "png" Then
            ExtractDataBox.Text = ExtractDataBox.Text & "The imagefile must be converted to a PNG-file." & EndLine
        End If

        '- Retrieving/extracting type of coordinate system (jprdata.gs)
        For LinePointer = Block(2).FirstLine To Block(2).LastLine
            If InStr(GeoDataLine(LinePointer), "CS[", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), "[", CompareMethod.Binary)
                JprData.gs = Strings.Right(GeoDataLine(LinePointer), Len(GeoDataLine(LinePointer)) - ChrPointer)
                ChrPointer = InStr(JprData.gs, ",", CompareMethod.Binary)
                JprData.gs = LCase(Strings.Left(JprData.gs, ChrPointer - 1))
                ExtractDataBox.Text = ExtractDataBox.Text & "coordinate system = " & JprData.gs & " (not a jpr variable)" & EndLine
            End If

        Next

        '- Retrieving/extracting unit of coordinate system (jprdata.gu), only if cartesian
        '  returns "meter" incase metre or meter (else "")
        If JprData.gs = "cartesian" Then
            OrderCounter = 0
            For LinePointer = Block(2).FirstLine To Block(2).LastLine
                If InStr(GeoDataLine(LinePointer), "ORDER[1]", CompareMethod.Binary) > 0 And
                        InStr(LCase(GeoDataLine(LinePointer + 1)), "met", CompareMethod.Binary) > 0 Then       '"met" stands for "metre" or "meter"
                    OrderCounter = OrderCounter + 1
                End If
                If InStr(GeoDataLine(LinePointer), "ORDER[2]", CompareMethod.Binary) > 0 And
                        InStr(LCase(GeoDataLine(LinePointer + 1)), "met", CompareMethod.Binary) > 0 Then       '"met" stands for "metre" or "meter"
                    OrderCounter = OrderCounter + 1
                End If
            Next

            If OrderCounter = 2 Then
                JprData.gu = "meter"
                ExtractDataBox.Text = ExtractDataBox.Text & "coordinate system unit = " & JprData.gu & " (not a jpr variable)" & EndLine
            End If

        End If

        '- Retrieving/extracting size from Block(0)
        For LinePointer = Block(0).FirstLine To Block(0).LastLine
            If InStr(GeoDataLine(LinePointer), "Size is", CompareMethod.Binary) > 0 Then
                TempString = Strings.Right(GeoDataLine(LinePointer), Len(GeoDataLine(LinePointer)) - 7)
                Dim Size() As String = Split(TempString, ",", CompareMethod.Binary)
                ImageWidth = Val(Size(0))
                ImageHeight = Val(Size(1))
            End If

        Next

        ExtractDataBox.Text = ExtractDataBox.Text & "image width = " & ImageWidth & " (not a jpr variable)" & EndLine
        ExtractDataBox.Text = ExtractDataBox.Text & "image height = " & ImageHeight & " (not a jpr variable)" & EndLine

        '- Setting Pixel coordinates of the upper left, upper right, lower right, lower left and center
        ReferencePoint(1).x = 0
        ReferencePoint(1).y = 0
        ReferencePoint(2).x = ImageWidth - 1
        ReferencePoint(2).y = 0
        ReferencePoint(3).x = ImageWidth - 1
        ReferencePoint(3).y = ImageHeight - 1
        ReferencePoint(4).x = 0
        ReferencePoint(4).y = ImageHeight - 1
        ReferencePoint(5).x = CommaToDot(ImageWidth * 0.5 - 1)      'to be sure the separator will be a dot
        ReferencePoint(5).y = CommaToDot(ImageHeight * 0.5 - 1)     'to be sure the separator will be a dot

        '- Setting Geigraphical coordinates (lat/lon and East/north) of the upper left, upper right, lower right, lower left and center
        For LinePointer = Block(7).FirstLine To Block(7).LastLine
            '- Retrieving and setting the geographical coordinate of the upper left corner
            If InStr(GeoDataLine(LinePointer), "Upper Left", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(GeoDataLine(LinePointer), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                Dim TempString3() As String = Split(TempString1(0), "(", CompareMethod.Binary)
                Dim TempString4() As String = Split(TempString3(1), ",", CompareMethod.Binary)
                ReferencePoint(1).lon = CommaToDot(CvtLatLonStr(TempString2(0)))
                ReferencePoint(1).lat = CommaToDot(CvtLatLonStr(Strings.Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
                ReferencePoint(1).east = Trim(TempString4(0))
                ReferencePoint(1).north = Trim(TempString4(1))
            End If

            '- Retrieving and setting the coordinates of the upper right corner
            If InStr(GeoDataLine(LinePointer), "Upper Right", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(GeoDataLine(LinePointer), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                Dim TempString3() As String = Split(TempString1(0), "(", CompareMethod.Binary)
                Dim TempString4() As String = Split(TempString3(1), ",", CompareMethod.Binary)
                ReferencePoint(2).lon = CommaToDot(CvtLatLonStr(TempString2(0)))
                ReferencePoint(2).lat = CommaToDot(CvtLatLonStr(Strings.Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
                ReferencePoint(2).east = Trim(TempString4(0))
                ReferencePoint(2).north = Trim(TempString4(1))
            End If

            '- Retrieving and setting the coordinates of the lower right corner
            If InStr(GeoDataLine(LinePointer), "Lower Right", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(GeoDataLine(LinePointer), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                Dim TempString3() As String = Split(TempString1(0), "(", CompareMethod.Binary)
                Dim TempString4() As String = Split(TempString3(1), ",", CompareMethod.Binary)
                ReferencePoint(3).lon = CommaToDot(CvtLatLonStr(TempString2(0)))
                ReferencePoint(3).lat = CommaToDot(CvtLatLonStr(Strings.Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
                ReferencePoint(3).east = Trim(TempString4(0))
                ReferencePoint(3).north = Trim(TempString4(1))
            End If

            '- Retrieving and setting the coordinates of the lower left corner
            If InStr(GeoDataLine(LinePointer), "Lower Left", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(GeoDataLine(LinePointer), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                Dim TempString3() As String = Split(TempString1(0), "(", CompareMethod.Binary)
                Dim TempString4() As String = Split(TempString3(1), ",", CompareMethod.Binary)
                ReferencePoint(4).lon = CommaToDot(CvtLatLonStr(TempString2(0)))
                ReferencePoint(4).lat = CommaToDot(CvtLatLonStr(Strings.Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
                ReferencePoint(4).east = Trim(TempString4(0))
                ReferencePoint(4).north = Trim(TempString4(1))
            End If

            '- Retrieving and setting the coordinates of the center
            If InStr(GeoDataLine(LinePointer), "Center", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(GeoDataLine(LinePointer), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                Dim TempString3() As String = Split(TempString1(0), "(", CompareMethod.Binary)
                Dim TempString4() As String = Split(TempString3(1), ",", CompareMethod.Binary)
                ReferencePoint(5).lon = CommaToDot(CvtLatLonStr(TempString2(0)))
                ReferencePoint(5).lat = CommaToDot(CvtLatLonStr(Strings.Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
                ReferencePoint(5).east = Trim(TempString4(0))
                ReferencePoint(5).north = Trim(TempString4(1))
            End If

        Next

        '- Copying reference points to 'ExtractDataBox' (only lat/log and pixel)
        For Pointer = 1 To 5
            ExtractDataBox.Text = ExtractDataBox.Text & "rp" & Pointer & " (Reference point " & Pointer & ") = " &
            ReferencePoint(Pointer).lat & ", " & ReferencePoint(Pointer).lon & ", " &
            ReferencePoint(Pointer).x & ", " & ReferencePoint(Pointer).y & EndLine
        Next

        '- Retrieving PixelSize based on reference points (only if coordinate system is carterisan)
        If JprData.gs = "cartesian" Then
            JprData.ps = PixelSize(ReferencePoint(1), ReferencePoint(2), ReferencePoint(3), ReferencePoint(4))
            ExtractDataBox.Text = ExtractDataBox.Text & "pixel size (in CS units) = " & JprData.ps & " (not a jpr variable)" & EndLine
        End If

        '- Handling the boundary
        '  MrSiD does not have a collar. The first four Rererencepoints wil be used.

        For Pointer = 1 To VertexAmount
            VertexPoint(Pointer) = ReferencePoint(Pointer)
            ExtractDataBox.Text = ExtractDataBox.Text & "vp" & Pointer & " (Vertex point " & Pointer & ") = " &
                CommaToDot(CStr(VertexPoint(Pointer).x)) & ", " &
                CommaToDot(CStr(VertexPoint(Pointer).y)) & EndLine
        Next

        '- Retrieving JprData.sr
        If JprData.it = "sid" Then                              'Case resolution from WKT in MrSID-file
            For LinePointer = Block(6).FirstLine To Block(6).LastLine
                If InStr(GeoDataLine(LinePointer), "DPI", CompareMethod.Binary) > 0 Then
                    Dim TempString1() As String = Split(GeoDataLine(LinePointer), "=", CompareMethod.Binary)
                    JprData.sr = TempString1(1)
                    ExtractDataBox.Text = ExtractDataBox.Text & "sr (resolution from SID) = " & JprData.sr & EndLine
                    DpiBox.Text = JprData.sr

                End If
            Next

        Else
            'Case resolution from image file 
            XResolution = ImageData.ResolutionX              'Retrieving X resolution
            YResolution = ImageData.ResolutionY              'Retrieving Y resolution

            If XResolution = YResolution And XResolution <> 0 Then
                JprData.sr = Math.Round(XResolution)
                ExtractDataBox.Text = ExtractDataBox.Text & "sr (resolution from image) = " & JprData.sr & EndLine
                DpiBox.Text = JprData.sr

            End If

        End If

        If JprData.sr = 0 Then
            ExtractDataBox.Text = ExtractDataBox.Text & "sr (resolution) couldn't retrieved." & EndLine
        End If

        '- Setting dotsize (ds) 

        If JprData.gs = "cartesian" And JprData.gu = "meter" Then
            JprData.ds = JprData.ps
            ExtractDataBox.Text = ExtractDataBox.Text & "pixel size (in meter) = " & CommaToDot(Math.Round(JprData.ds, 5)) & " (not a jpr variable)" & EndLine

        Else
            JprData.ds = 0

        End If

        '- Retrieving/extracting geographical datum and projection data from Block(2) (base- or only CRS) 
        For LinePointer = Block(2).FirstLine To Block(2).LastLine

            '- Retrieving/extracting dm (geographic datum)
            If InStr(GeoDataLine(LinePointer), "DATUM", CompareMethod.Binary) > 0 Then
                Dim DatumStr() As String = Split(GeoDataLine(LinePointer), "[", CompareMethod.Binary)
                JprData.dm = Strings.Mid(DatumStr(1), 2, Len(DatumStr(1)) - 3)
                JprData.dm = TranslateDatum(JprData.dm)                     'Harmonizing Datum
                ExtractDataBox.Text = ExtractDataBox.Text & "dm (datum) = " & JprData.dm & EndLine
            End If

            '- Retrieving/extracting pr (projection)
            If InStr(GeoDataLine(LinePointer), "METHOD", CompareMethod.Binary) > 0 Then
                Dim PrStr() As String = Split(GeoDataLine(LinePointer), "[", CompareMethod.Binary)
                JprData.pr = Strings.Mid(PrStr(1), 2, Len(PrStr(1)) - 3)
            End If

            '- Retrieving/extracting pp ('central meridian' or 'Longitude of natural origin')
            If InStr(GeoDataLine(LinePointer), "Longitude of natural origin", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.pp = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "pp (central meridian/Longitude of origin) = " & JprData.pp & EndLine
            End If

            '- Retrieving/extracting p1 ('Latitude of natural origin')
            If InStr(GeoDataLine(LinePointer), "Latitude of natural origin", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p1 = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p1 (Latitude of origin) = " & JprData.p1 & EndLine
            End If

            '- Retrieving/extracting p2 (scale factor)
            If InStr(GeoDataLine(LinePointer), "Scale factor at natural origin", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p2 = Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p2 (scale factor at origin) = " & JprData.p2 & EndLine
            End If

            '- Retrieving/extracting p3 (false northing)
            If InStr(GeoDataLine(LinePointer), "False northing", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p3 = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p3 (false northing) = " & JprData.p3 & EndLine
            End If

            '- Retrieving/extracting p4 (false easting)
            If InStr(GeoDataLine(LinePointer), "False easting", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p4 = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p4 (false easting) = " & JprData.p4 & EndLine
            End If

            '- Retrieving/extracting p5 (standard parallel 1)
            If InStr(GeoDataLine(LinePointer), "Latitude of 1st standard parallel", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p5 = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p5 (standard parallel 1) = " & JprData.p5 & EndLine
            End If

            '- Retrieving/extracting p6 (standard parallel 2)
            If InStr(GeoDataLine(LinePointer), "Latitude of 2nd standard parallel", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr(GeoDataLine(LinePointer), ",", CompareMethod.Binary)
                JprData.p6 = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 1, Len(GeoDataLine(LinePointer)) - ChrPointer - 1)
                ExtractDataBox.Text = ExtractDataBox.Text & "p6 (standard parallel 2) = " & JprData.p6 & EndLine
            End If


            '  - Retrieving/extracting UTM zone (UTM zone data) 
            If InStr(GeoDataLine(LinePointer), "UTM zone", CompareMethod.Binary) > 0 And
                InStr(GeoDataLine(LinePointer), "CONVERSION", CompareMethod.Binary) > 0 Then
                ChrPointer = InStr((GeoDataLine(LinePointer)), "UTM zone", CompareMethod.Binary)
                TempString = Strings.Mid(GeoDataLine(LinePointer), ChrPointer + 8, 4)
                If Strings.Right(TempString, 1) = Chr(34) Then
                    TempString = "0" & Strings.Mid(TempString, 2, 2)
                Else
                    TempString = Strings.Trim(TempString)
                End If
                JprData.zn = TempString
                Select Case Strings.Mid(JprData.zn, 3, 1)
                    Case "N", "n"
                        JprData.zn = Strings.Mid(JprData.zn, 1, 2) & "t"
                    Case "S", "s"
                        JprData.zn = Strings.Mid(JprData.zn, 1, 2) & "j"
                    Case Else
                        '- if no N or S: the t or j will be based on latitude of center map (reference point 5)
                        If Val(CommaToDot(CStr(ReferencePoint(5).lat))) >= 0 Then
                            JprData.zn = Strings.Mid(JprData.zn, 1, 2) & "t"
                        Else
                            JprData.zn = Strings.Mid(JprData.zn, 1, 2) & "j"
                        End If
                End Select
                ExtractDataBox.Text = ExtractDataBox.Text & "zn (UTM zone) = " & JprData.zn & EndLine

            End If

        Next

        '- Harmonizing Projection
        JprData.pr = TranslateProjection(JprData.pr, JprData.zn)
        ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & EndLine

        '- Enabling next steps in the application and moving text cursor to ExtractDataBox
        ExtractGroup.Enabled = True
        ExtractDataBox.Select()
        ExtractDataBox.DeselectAll()
        BuildItem.Enabled = True
        BuildItem.Select()

        '- Disabling SaveItem
        SaveItem.Enabled = False

    End Sub

    'Subroutine BuildItem_Click initiates the correct content for the JPR-file
    Private Sub BuildItem_Click(sender As Object, e As EventArgs) Handles BuildItem.Click

        '- Handling Scale/resolution and DpiBox
        If JprData.ds > 0 Then
            DpiBox.Enabled = True
        Else
            DpiBox.Enabled = False
        End If

        '- Calling function BuildJprContent (which genrates the content for the JPR-File) 
        JprDataBox.Text = BuildJprContent("ini")

        '- Enabling next step in the application and moving text cursor to DataBox
        JprGroup.Enabled = True
        JprDataBox.Select()
        JprDataBox.DeselectAll()
        SaveItem.Enabled = True
        SaveItem.Select()

    End Sub

    'Function BuildJprContent generates the content for the JPR-file (as text string)
    Private Function BuildJprContent(Trigger As String)
        '- Declarations
        Dim Pointer As Integer          'Subroutine internal variable; used as an array pointer
        Dim ContentText As String       'Subroutine internal variable; represents the content for the JPR-File

        '- Starting with a commentline in ContentText
        ContentText = "//This file was created using M4MM " & Version & " (Maps for Memory Map) by www.hzns.nl" & EndLine &
            "nm=" & JprData.nm & EndLine

        Select Case JprData.dm
            Case "wgs84", "nad27", "nad83", "osgb"
                ContentText = ContentText & "dm=" & JprData.dm & EndLine
            Case ""
                ContentText = ContentText & "//no datum (dm) was found." & EndLine
            Case Else
                ContentText = ContentText & "//found datum (dm) " & JprData.dm & " isnt supported." & EndLine
        End Select

        '- Setting Shift data
        ContentText = ContentText &
            "st=" & "0" & EndLine &
            "sn=" & "0" & EndLine

        '- Handling projection specific variables
        ContentText = ContentText & "pr=" & JprData.pr & EndLine

        Select Case JprData.pr
            Case "utm"
                ContentText = ContentText & "zn=" & JprData.zn & EndLine
            Case "mercator"
                ContentText = ContentText & "p1=" & JprData.p1 & EndLine
            Case "transverse mercator"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine
            Case "lambert conformal conic"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
            Case Else
                ContentText = ContentText & "//This projection is not supported by Maps for Memory-Map (M4MM), please check and edit 'p-data'" & EndLine &
                    "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
        End Select

        '- Handling resolution and scale variables
        If JprData.ds > 0 Then
            Select Case Trigger
                Case "ini"
                    JprData.sr = Val(DpiBox.Text)
                    JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                    'Writing dat to ScaleBox
                    ScaleBox.Text = JprData.sc

                Case "dpi"
                    JprData.sr = Val(DpiBox.Text)
                    JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                    'Writing dat to ScaleBox
                    ScaleBox.Text = JprData.sc

                Case "scale"
                    JprData.sc = Val(ScaleBox.Text)
                    JprData.sr = Int(JprData.sc * 2.54 / (JprData.ds * 100))

                    'Writing dat to ScaleBox
                    DpiBox.Text = JprData.sr

            End Select

            ContentText = ContentText & "sr=" & JprData.sr & EndLine
            ContentText = ContentText & "sc=1:" & JprData.sc & EndLine
        Else
            ContentText = ContentText & "sr=" & JprData.sr & EndLine

        End If

        '- Handling graphic variables
        If JprData.it <> "png" Then
            ContentText = ContentText & "//You are advised to convert the image to PNG and edit this JPR file (it=png)." & EndLine
        End If

        ContentText = ContentText & "it=" & JprData.it & EndLine

        '- Handling the color depth 
        If ImageData.Valid = True And ImageData.Usable = False Then
            ContentText = ContentText & "//The colordepth is incompatible with Memory-Map. Please convert to 8 bit/256 colors of less." & EndLine

        End If


        '- Handling reference points
        For Pointer = 1 To 5
            ContentText = ContentText & "rp" & Pointer & "=" &
                CommaToDot(ReferencePoint(Pointer).lat) & "," &
                CommaToDot(ReferencePoint(Pointer).lon) & "," &
                ReferencePoint(Pointer).x & "," &
                ReferencePoint(Pointer).y & EndLine
        Next

        '- Handling boundary points
        For Pointer = 1 To VertexPoint.Length - 1
            ContentText = ContentText & "vp" & Pointer & "=" &
            VertexPoint(Pointer).x & "," &
            VertexPoint(Pointer).y & EndLine
        Next

        Return ContentText

    End Function

    'Subroutine ClearData clears all the information of variables used to create a JPR file
    Private Sub ClearData()
        '- Declarations
        Dim Pointer As Integer          'Subroutine internal variable; used as an array pointer

        '- The JPR data
        JprData = ClearJprData()

        '-  The necessary Resdata
        ResData = ClearResolution()

        '- The reference points
        For Pointer = 1 To 5
            ReferencePoint(Pointer) = ClearCoordinate()
        Next

        '- The boundary (like Neatline)
        For Pointer = 1 To VertexAmount
            VertexPoint(Pointer) = ClearCoordinate()
        Next
        ReDim VertexPoint(4)

    End Sub

    'Subroutine DpiBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If

        End If

    End Sub

    'Subroutine DpiBox_PressKey handles the case "Enter"-key (chr13) was used (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprData.sr = Val(DpiBox.Text)
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If

        End If

    End Sub

    'Subroutine ScaleBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_LostFocus(sender As Object, e As EventArgs) Handles ScaleBox.LostFocus, ScaleBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()

            End If

        End If

    End Sub

    'Subroutine ScaleBox_PressKey handles the case "Enter"-key was used (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles ScaleBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()

            End If

        End If

    End Sub

    'Subroutine SaveItem_Click 
    '-  Saving the JrpDataBox.text to a JPR-file in the same directory as the orginal MAP file. 
    '   If this JPR file exists a warning will be prompted.
    Private Sub SaveItem_Click(sender As Object, e As EventArgs) Handles SaveItem.Click
        '- Declarations
        Dim Answer As Integer           'Subroutine internal variable; represents answer value for MsgBox

        'Checking for an existing file
        If My.Computer.FileSystem.FileExists(JprFile.LongName) = True Then
            '(Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
            Answer = MsgBox("JPR-file allready exits. Overwrite?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox = Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
            End If
        Else
            'Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
        End If

    End Sub

    'Subroutine PrintDataItem_Click
    '- Initiating Printing GeoDataBox.text with color data
    Private Sub PrintDataItem_Click(sender As Object, e As EventArgs) Handles PrintDataItem.Click

        StartMenu.PrintContent(GeoDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine PrintDataNoClrItem_Click
    '- Initiating Printing GeoDataBox.text without color data
    Private Sub PrintDataNoClrItem_Click(sender As Object, e As EventArgs) Handles PrintDataNoClrItem.Click

        '- Trimming color data by
        '  splitting into lines 
        '  cycle through lines until "band "  
        Dim TempLine() As String = Split(GeoDataBox.Text, EndLine)
        Dim TempPointer As Integer
        Dim TempText As String = ""

        For TempPointer = 0 To TempLine.Count - 1
            If LCase(Strings.Left(TempLine(TempPointer), 5)) <> "band " Then
                TempText = TempText & TempLine(TempPointer) & EndLine
            Else
                Exit For
            End If
        Next

        StartMenu.PrintContent(TempText, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine ExtractedDataItem_Click
    '- Initiating Printing ExtractDataBox.text without color data
    Private Sub ExtractedDataItem_Click(sender As Object, e As EventArgs) Handles ExtractedDataItem.Click

        StartMenu.PrintContent(ExtractDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine JprDataItem_Click_Click
    '- Initiating Printing JprDataBox.text without color data
    Private Sub JprDataItem_Click(sender As Object, e As EventArgs) Handles JprDataItem.Click

        StartMenu.PrintContent(JprDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine PrintItem_Click
    '- Testing if there is something to print (ExtractDataBox and JprDataBox)
    Private Sub PrintItem_Click(sender As Object, e As EventArgs) Handles PrintItem.Click

        '- Enabling/disabling printing ExtractDataBox
        If Trim(ExtractDataBox.Text) = "" Then
            ExtractedDataItem.Enabled = False
        Else
            ExtractedDataItem.Enabled = True
        End If

        '- Enabling/disabling printing JprDataBox
        If Trim(JprDataBox.Text) = "" Then
            JprDataItem.Enabled = False
        Else
            JprDataItem.Enabled = True
        End If

    End Sub
End Class