﻿Imports System.ComponentModel

Public Class SidConv

    'Declarations
    Public SourceFile As FileNameStructure          'Variable; represents the SID source-filename (image)
    Public TargetFile As FileNameStructure          'Variable; represents the target-filename (result image)


    'Subroutine SidConv_Load
    '- Actions at opening the form SidConv
    Private Sub MrSidConv_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "Select a MrSID-file to view the content. If a file is selected the corresponding OZF2/OZFX3- file can be converted.")
        TipText.SetToolTip(SourceFileValue, "Filename and location of the MrSID-file.")
        TipText.SetToolTip(CommentBox, "Information about the coversion process.")
        TipText.SetToolTip(TargetFileValue, "Filename and location for the converted MrSID-file.")
        TipText.SetToolTip(BottomStrip, "Used Gdal version.")

        '- Clearing all values in form
        SourceFileValue.Text = ""
        TargetFileValue.Text = ""
        CommentBox.Text = ""

        '- Disabling StartForm form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Convert MrSID-file"

        '- Disabling TopStrip items (until item can be used)
        ConvertItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form MrSidConv
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form MrSidConv
    Private Sub MrSidConv_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling StartMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectItem_Click
    '- selecting a MrSID-file
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click

        '- Clearing all values in form
        SourceFileValue.Text = ""
        TargetFileValue.Text = ""
        GdalVersion.Text = "GDAL version"

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "MrSID-file (*.sid)|*.sid"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            TargetFileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")
            CommentBox.Text = "No sourcefile was selected" & EndLine

            '- Disabling TopStrip items (until item can be used)
            ConvertItem.Enabled = False

            Exit Sub

        End If


        SourceFile = SplitFileName(SelectFile.FileName)
        TargetFile = SplitFileName(SourceFile.Path & SourceFile.ShortName & ".png")

        '- Showing selected source and target file in form
        SourceFileValue.Text = SourceFile.LongName
        TargetFileValue.Text = TargetFile.LongName
        CommentBox.Text = "Sourcefile " & SourceFile.Name & " was selected." & EndLine

        '- Enabling next step
        ConvertItem.Enabled = True

    End Sub

    'Subroutine ConvertItem_click
    '- Converting a MrSID-file into a TIFF-file
    Private Sub ConvertItem_Click(sender As Object, e As EventArgs) Handles ConvertItem.Click
        '- Declarations     
        Dim CommandLine As String                   'Subroutine internal variable; represents string to be executed shell commandline

        '- Building the CommandLine
        CommandLine = BatFile.LongName & " Gdal_Translate -ot byte " &
            C34 & SourceFile.LongName & C34 & " " &
            C34 & TargetFile.LongName & C34

        '- Executing the CommandLine for converting Ozi file
        Call Shell(CommandLine, AppWinStyle.NormalFocus, Wait:=True)
        '- Writing information to the CommentBox
        CommentBox.Text = CommentBox.Text & "PNG-File (" & TargetFile.Name & ") from " & SourceFile.Name & " was created." & EndLine

        '- Retrieving used GDAL version using function UsedGdal and writing to GdalVersion in BottomStrip
        GdalVersion.Text = "Used GDAL version: " & UsedGdal()

    End Sub

End Class